@@ -39,6 +39,7 @@ filter_testcase basic basic-filename --invert-paths --path-glob 't*en*'
 filter_testcase basic basic-numbers  --invert-paths --path-regex 'f.*e.*e'
 filter_testcase basic basic-mailmap  --mailmap ../t9390/sample-mailmap
 filter_testcase basic basic-replace  --replace-text ../t9390/sample-replace
+filter_testcase basic basic-message  --replace-message ../t9390/sample-message
 filter_testcase empty empty-keepme   --path keepme
 filter_testcase empty more-empty-keepme --path keepme --prune-empty=always \
 		                                   --prune-degenerate=always
@@ -186,6 +187,19 @@ test_expect_success '--paths-from-file' '
 	)
 '
 
+test_expect_success '--paths does not mean --paths-from-file' '
+	setup_path_rename &&
+	(
+		git clone file://"$(pwd)"/path_rename paths_misuse &&
+		cd paths_misuse &&
+
+		test_must_fail git filter-repo --paths values/large 2>../err &&
+
+		grep "Error: Option.*--paths.*unrecognized; did you" ../err &&
+		rm ../err
+	)
+'
+
 create_path_filtering_and_renaming() {
 	test -d path_filtering_and_renaming && return
 
@@ -531,6 +545,22 @@ test_expect_success FUNNYNAMES 'creation/deletion/updating of replace refs' '
 		echo "$(git rev-parse master~1) refs/replace/$master_2" >>out &&
 		sort -k 2 out >expect &&
 		git show-ref | grep refs/replace/ >output &&
+		test_cmp output expect &&
+
+		rsync -a --delete ../replace_handling/ ./ &&
+		git filter-repo --replace-refs old-default --path-rename numbers:counting &&
+		echo "$(git rev-parse master) refs/replace/$master" >>out &&
+		echo "$(git rev-parse master~1) refs/replace/$master_1" >>out &&
+		echo "$(git rev-parse master~1) refs/replace/$master_2" >>out &&
+		sort -k 2 out >expect &&
+		git show-ref | grep refs/replace/ >output &&
+		test_cmp output expect &&
+
+		# Test the default
+		rsync -a --delete ../replace_handling/ ./ &&
+		git filter-repo --path-rename numbers:counting &&
+		echo "$(git rev-parse master~1) refs/replace/$master_1" >expect &&
+		git show-ref | grep refs/replace/ >output &&
 		test_cmp output expect
 	)
 '
@@ -722,8 +752,11 @@ test_expect_success C_LOCALE_OUTPUT '--analyze' '
 
 		git filter-repo --analyze &&
 
-		# It should work and overwrite report if run again
-		git filter-repo --analyze &&
+		# It should not work again without a --force
+		test_must_fail git filter-repo --analyze &&
+
+		# With a --force, another run should succeed
+		git filter-repo --analyze --force &&
 
 		test -d .git/filter-repo/analysis &&
 		cd .git/filter-repo/analysis &&
@@ -824,6 +857,38 @@ test_expect_success C_LOCALE_OUTPUT '--analyze' '
 	)
 '
 
+test_expect_success C_LOCALE_OUTPUT '--analyze --report-dir' '
+	setup_analyze_me &&
+	(
+		cd analyze_me &&
+
+		rm -rf .git/filter-repo &&
+		git filter-repo --analyze --report-dir foobar &&
+
+		# It should not work again without a --force
+		test_must_fail git filter-repo --analyze --report-dir foobar &&
+
+		# With a --force, though, it should overwrite
+		git filter-repo --analyze --report-dir foobar --force &&
+
+		test ! -d .git/filter-repo/analysis &&
+		test -d foobar &&
+
+		cd foobar &&
+
+		# Very simple tests because already tested above.
+		test_path_is_file renames.txt &&
+		test_path_is_file README &&
+		test_path_is_file blob-shas-and-paths.txt &&
+		test_path_is_file directories-all-sizes.txt &&
+		test_path_is_file directories-deleted-sizes.txt &&
+		test_path_is_file extensions-all-sizes.txt &&
+		test_path_is_file extensions-deleted-sizes.txt &&
+		test_path_is_file path-all-sizes.txt &&
+		test_path_is_file path-deleted-sizes.txt
+	)
+'
+
 test_expect_success '--replace-text all options' '
 	setup_analyze_me &&
 	(
@@ -849,6 +914,87 @@ test_expect_success '--replace-text all options' '
 	)
 '
 
+test_expect_success '--replace-text binary zero_byte-0_char' '
+	(
+		set -e
+		set -u
+		REPO=replace-text-detect-binary
+		FILE=mangle.bin
+		OLD_STR=replace-from
+		NEW_STR=replace-with
+		# used with printf, contains a zero byte and a "0" character, binary
+		OLD_CONTENT_FORMAT="${OLD_STR}\\0${OLD_STR}\\n0\\n"
+		# expect content unchanged due to binary
+		NEW_CONTENT_FORMAT="${OLD_CONTENT_FORMAT}"
+
+		rm -rf "${REPO}"
+		git init "${REPO}"
+		cd "${REPO}"
+		echo "${OLD_STR}==>${NEW_STR}" >../replace-rules
+		printf "${NEW_CONTENT_FORMAT}" > ../expect
+		printf "${OLD_CONTENT_FORMAT}" > "${FILE}"
+		git add "${FILE}"
+		git commit -m 'test'
+		git filter-repo --force --replace-text ../replace-rules
+
+		test_cmp ../expect "${FILE}"
+	)
+'
+
+test_expect_success '--replace-text binary zero_byte-no_0_char' '
+	(
+		set -e
+		set -u
+		REPO=replace-text-detect-binary
+		FILE=mangle.bin
+		OLD_STR=replace-from
+		NEW_STR=replace-with
+		# used with printf, contains a zero byte but no "0" character, binary
+		OLD_CONTENT_FORMAT="${OLD_STR}\\0${OLD_STR}\\n"
+		# expect content unchanged due to binary
+		NEW_CONTENT_FORMAT="${OLD_CONTENT_FORMAT}"
+
+		rm -rf "${REPO}"
+		git init "${REPO}"
+		cd "${REPO}"
+		echo "${OLD_STR}==>${NEW_STR}" >../replace-rules
+		printf "${NEW_CONTENT_FORMAT}" > ../expect
+		printf "${OLD_CONTENT_FORMAT}" > "${FILE}"
+		git add "${FILE}"
+		git commit -m 'test'
+		git filter-repo --force --replace-text ../replace-rules
+
+		test_cmp ../expect "${FILE}"
+	)
+'
+
+test_expect_success '--replace-text text-file no_zero_byte-zero_char' '
+	(
+		set -e
+		set -u
+		REPO=replace-text-detect-binary
+		FILE=mangle.bin
+		OLD_STR=replace-from
+		NEW_STR=replace-with
+		# used with printf, contains no zero byte but contains a "0" character, text
+		OLD_CONTENT_FORMAT="${OLD_STR}0\\n0${OLD_STR}\\n0\\n"
+		# expect content changed due to text
+		NEW_CONTENT_FORMAT="${NEW_STR}0\\n0${NEW_STR}\\n0\\n"
+
+		rm -rf "${REPO}"
+		git init "${REPO}"
+		cd "${REPO}"
+		echo "${OLD_STR}==>${NEW_STR}" >../replace-rules
+		printf "${NEW_CONTENT_FORMAT}" > ../expect
+		printf "${OLD_CONTENT_FORMAT}" > "${FILE}"
+		git add "${FILE}"
+		git commit -m 'test'
+		git filter-repo --force --replace-text ../replace-rules
+
+		test_cmp ../expect "${FILE}"
+	)
+'
+
 test_expect_success '--strip-blobs-bigger-than' '
 	setup_analyze_me &&
 	(
@@ -1344,6 +1490,46 @@ test_expect_success '--target' '
 	test_cmp target/expect target/actual
 '
 
+test_expect_success '--date-order' '
+	test_create_repo date_order &&
+	(
+		cd date_order &&
+		git fast-import --quiet <$DATA/date-order &&
+		# First, verify that without date-order, C is before B
+		cat <<-EOF >expect-normal &&
+		Initial
+		A
+		C
+		B
+		D
+		merge
+		EOF
+		git filter-repo --force --message-callback "
+			with open(\"messages.txt\", \"ab\") as f:
+				f.write(message)
+			return message
+			" &&
+		test_cmp expect-normal messages.txt &&
+
+		# Next, verify that with date-order, C and B are reversed
+		rm messages.txt &&
+		cat <<-EOF >expect &&
+		Initial
+		A
+		B
+		C
+		D
+		merge
+		EOF
+		git filter-repo --date-order --force --message-callback "
+			with open(\"messages.txt\", \"ab\") as f:
+				f.write(message)
+			return message
+			" &&
+		test_cmp expect messages.txt
+	)
+'
+
 test_expect_success '--refs' '
 	setup_analyze_me &&
 	git init refs &&
@@ -1451,7 +1637,7 @@ test_expect_success 'handle funny characters' '
 
 		file_sha=$(git rev-parse :0:señor) &&
 		former_head_sha=$(git rev-parse HEAD) &&
-		git filter-repo --to-subdirectory-filter títulos &&
+		git filter-repo --replace-refs old-default --to-subdirectory-filter títulos &&
 
 		cat <<-EOF >expect &&
 		100644 $file_sha 0	"t\303\255tulos/se\303\261or"
